//  COPYRIGHT:       PrimeObjects Software Inc. (C) 2022 All Right Reserved
//  COMPANY URL:     https://www.primeobjects.com/
//  CONTACT:         developer@primeobjects.com
//
//  This source is subject to the PrimeObjects License Agreements.
import React, { useEffect } from "react";
import { isNil } from "lodash";
import { libTrackIsOn, isNonEmptyString } from "primeobjects-helper-util";
import { useRef, useState } from "react";
import { useServerSentEvents } from "primeobjects-ui-web-basic/build/cjs/hooks/use-sse";
import TextareaAutosize from "react-autosize-textarea/lib/index";
import { getCurrentPoolUser } from "../context/auth-cognito";
import { useSolution } from "primeobjects-helper-ui/build/cjs/hooks/solution";
import { callApi } from "../context/auth-call-api";
export var AiStream = function (props) {
    var _a;
    var style = props.style, placeholder = props.placeholder, disabled = props.disabled, rows = props.rows, maxRows = props.maxRows, assistant = props.assistant;
    var className = (_a = props.className) !== null && _a !== void 0 ? _a : "";
    var textRef = useRef("");
    var _b = useState(undefined), streaming = _b[0], setStreaming = _b[1];
    var _c = useState(""), value = _c[0], setValue = _c[1];
    var start = props.start ? props.start : assistant === null || assistant === void 0 ? void 0 : assistant.start;
    var solution = useSolution();
    var appendValue = function (v) {
        textRef.current = textRef.current + v;
        setValue(textRef.current);
    };
    var resetValue = function () {
        textRef.current = "";
        setValue("");
    };
    var onError = function (error) {
        console.error(error);
        setStreaming(undefined);
        props.onError && props.onError(error);
    };
    var openStream = useServerSentEvents({
        baseUrl: "/api/stream",
        config: {
            withCredentials: false
        },
        onData: function (data) {
            var _a;
            if (libTrackIsOn())
                console.log({ onData: data });
            try {
                var text = (_a = JSON.parse(data).choices[0].text) !== null && _a !== void 0 ? _a : JSON.parse(data).choices[0].delta.content;
                if (isNonEmptyString(text) &&
                    ((textRef.current.length == 0 && text != "\n") ||
                        textRef.current.length > 0)) {
                    appendValue(text);
                }
                props.onData && props.onData(text);
            }
            catch (err) {
                console.log("Failed to parse data: ".concat(data));
            }
        },
        onOpen: function () {
            if (libTrackIsOn())
                console.log("onOpen");
            props.onStart && props.onStart();
            // if (textRef.current) {
            //     textRef.current.innerHTML = "";
            // }
        },
        onClose: function () {
            if (libTrackIsOn())
                console.log("onClose");
            onFinish();
        },
        onError: function (event) {
            onError(event);
        }
    }).openStream;
    var onChange = function (e) {
        props.onChange && props.onChange(e.target.value);
    };
    var runStream = function (auth) {
        //because the stream request will fail if there is large content in the assistant
        //we create an S3 file first by calling submit-assistant-stream API, it contains assistant and OpenAI data
        //the stream will get detail from s3 file by calling start-assistant-stream
        callApi(solution, "".concat(solution.apis.ai, "submit-assistant-stream"), 
        //"http://localhost:3000/prod/finish-assistant-stream",
        {
            assistant: assistant
        }, "POST")
            .then(function (result) {
            openStream({
                query: {
                    auth: auth,
                    transactionId: result.transactionId
                }
            });
        })
            .catch(function (error) {
            onError(error);
        });
    };
    var onFinish = function () {
        var _a, _b, _c, _d, _e;
        setStreaming(undefined);
        var result = "".concat(textRef.current);
        props.onFinish && props.onFinish("".concat(textRef.current));
        var estimatedTokens = ("".concat((_a = assistant === null || assistant === void 0 ? void 0 : assistant.command) !== null && _a !== void 0 ? _a : "").length +
            "".concat((_b = assistant === null || assistant === void 0 ? void 0 : assistant.content) !== null && _b !== void 0 ? _b : "").length +
            ((_c = assistant === null || assistant === void 0 ? void 0 : assistant.maxResultLength) !== null && _c !== void 0 ? _c : 0)) /
            4;
        var realTokens = ("".concat((_d = assistant === null || assistant === void 0 ? void 0 : assistant.command) !== null && _d !== void 0 ? _d : "").length +
            "".concat((_e = assistant === null || assistant === void 0 ? void 0 : assistant.content) !== null && _e !== void 0 ? _e : "").length +
            result.length) /
            4;
        callApi(solution, "".concat(solution.apis.ai, "finish-assistant-stream"), 
        //"http://localhost:3000/prod/finish-assistant-stream",
        {
            model: assistant === null || assistant === void 0 ? void 0 : assistant.model,
            transactionId: assistant === null || assistant === void 0 ? void 0 : assistant.transactionId,
            tokens: realTokens - estimatedTokens
        }, "POST");
    };
    useEffect(function () {
        if (!isNil(start) && start != streaming) {
            resetValue();
            setStreaming(start);
            getCurrentPoolUser(props.solution ? props.solution : solution).then(function (cognito) {
                if (cognito) {
                    var authorization = cognito.signInUserSession.idToken.jwtToken;
                    var accessToken = cognito.signInUserSession.accessToken.jwtToken;
                    var auth = { authorization: authorization, accessToken: accessToken };
                    runStream(JSON.stringify(auth));
                }
            });
        }
    }, [start]);
    return (React.createElement(TextareaAutosize, { value: value, style: style, rows: rows, maxRows: maxRows, disabled: disabled, className: className, placeholder: placeholder, onChange: onChange }));
};
export default AiStream;
//# sourceMappingURL=ai-stream.js.map